"""This file is generated by setuptools-generate.
The information comes from pyproject.toml.
It provide some metainfo for docs/conf.py to build documents and
help2man to build man pages.
"""

# For docs/conf.py
project = "zathura-language-server"
author = """Wu, Zhenyu <wuzhenyu@ustc.edu> """
copyright = "2025"

# For help2man
DESCRIPTION = "language server for zathurarc"
EPILOG = "Report bugs to https://github.com/Freed-Wu/zathura-language-server/issues"
# format __version__ by yourself
VERSION = """zathura-language-server {__version__}
Copyright (C) 2025
Written by Wu, Zhenyu <wuzhenyu@ustc.edu> """
SOURCE = "https://github.com/Freed-Wu/zathura-language-server"